normal_wave_turns = {2,4,6,9,12,15,18}

-- *** Hex zones ***
-- Spawn zones
-- When spawning a set of units, the script will scan spawn zones hex by hex, in the order specified, and spawn units in free hexes it finds
-- If the script runs out of hexes, the remainder of the units will not be spawned.
ru_spawn_zone1 = {{15,1},{16,1},{17,1},{18,1},{19,1},{20,1},{21,1},{22,1},{16,2},{18,2},{20,2},{22,2},{23,2}}
ru_spawn_zone2 = {{36,1},{37,1},{38,1},{39,1},{40,1},{38,2},{39,2},{40,2},{41,2},{37,3},{38,3},{40,3},{41,3},{38,4},{39,4},{41,4},{39,5},{40,5}}
ru_spawn_zone3 = {{41,14},{42,14},{40,15},{41,15},{42,15},{42,16},{41,17},{42,17},{41,18},{42,18},{40,19},{42,19},{41,20},{42,20},{41,21}}
ru_spawn_zone4 = {{40,30},{41,30},{39,31},{41,31},{39,32},{40,32},{41,32},{38,33},{39,33},{36,34},{37,34},{38,34},{39,34},{35,35},{37,35},{36,36},{37,36}}
ru_spawn_zone5 = {{14,33},{15,33},{16,33},{14,34},{17,34},{13,35},{14,35},{15,35},{16,35},{17,35},{13,36},{14,36},{15,36},{16,36},{17,36},{18,36}}

-- *** Enemy waves ***
-- Each unit set is a list of blocks in the following format: { "<Unit ID>", "<Transport ID>", overstrength, experience }
-- If no organic transport is needed, its respective field should be empty string ("").

-- Soviet waves
ru_wave1 = { {"Regular", "", 1, 900}, {"SMGInf", "", 1, 900}, {"SUHWInf", "SUTruck", 2, 900}, {"T34-42", "", 1, 900}, {"BT7", "", 2, 900}, {"122mmM1938", "SUTruck", 2, 900} }
ru_wave2 = { {"Regular", "", 1, 900}, {"Regular", "", 1, 900}, {"SMGInf", "SUTruck", 2, 900}, {"T34-41", "", 1, 900}, {"BT7", "", 2, 900}, {"BM13Katyusha", "SUTruck", 2, 900} }
ru_wave3 = { {"Regular", "", 2, 900}, {"Regular", "", 2, 900}, {"SUHWInf", "SUTruck", 3, 900}, {"SMGInf", "", 1, 900}, {"SUMntInf", "", 1, 900}, {"45mmM1937", "SUTruck", 2, 900}, {"BA10", "", 1, 900} }
ru_wave4 = { {"KV1-41", "", 1, 900}, {"Conscript", "", 1, 900}, {"T50", "", 2, 900}, {"Conscript", "", 1, 900}, {"SMGInf", "", 2, 900}, {"T34-41", "", 2, 900}, {"37mmM1939AA", "SUTruck", 2, 900}, {"122mmM1938", "SUTruck", 2, 900} }
ru_wave5 = { {"Regular", "", 1, 900}, {"SMGInf", "", 1, 900}, {"T34-42", "", 1, 900}, {"Conscript", "", 2, 900}, {"BM13Katyusha", "", 1, 900}, {"GazAAMG-AA", "", 2, 900} }
ru_wave6 = { {"T34-57", "", 1, 1900}, {"T34-43", "", 1, 1900}, {"KV2TNK", "", 1, 1900}, {"Guards", "", 4, 2900}, {"BM13Katyusha", "", 1, 2900}, {"BA64", "", 2, 2900} }

-- There can be any number of waves in this list, the game will pick a random one every time
ru_waves = {ru_wave1, ru_wave2, ru_wave3, ru_wave4, ru_wave5, ru_wave6}

-- There can be any number of zones in this list, the game will pick a random one every time
ru_zones = {ru_spawn_zone1, ru_spawn_zone2, ru_spawn_zone3, ru_spawn_zone4, ru_spawn_zone5}

function SpawnWave(player, zone, units)
	local owner = world:GetPlayer(player)

	local cur_hex = 1

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = true
		create_action.cost = 0 -- don't take money for units we generate
		create_action.faction = owner.factions[1]  --set to first faction of owning player for created units
		world:Exec(create_action)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 70 }
		unit.aggressiveness = player_aggressiveness[player+1]
		
	end

end

function OnNewRound(action)
	-- On the first call, initialize the random generator
	if world.round == 1 then
		math.randomseed(world.seed)
	end

	-- Do we need to spawn a normal wave?
	local is_normal_wave = IsItemInTable(normal_wave_turns, world.round)

	-- If we don't need to spawn any normal or boss waves this turn then just finish
	if not is_normal_wave then return end

	-- Determine the player to get the next wave, always USSR 1 
	player = 1
	
	-- Determine the right zone and units to spawn
	local zone
	local units

	zone = ru_zones[ math.random( #ru_zones ) ] -- random normal ru zone
	units = ru_waves[ math.random( #ru_waves ) ] -- random normal ru wave
	SpawnWave(player, zone, units) -- Now spawn this wave!
end

-- *** Utility functions ***
function FindItemInTable(table, item)
	for index,i in ipairs(table) do
		if i == item then
			return index 
		end
	end

	return -1
end

function IsItemInTable(table, item)
	return FindItemInTable(table, item) ~= -1
end

function EmptyCondition()
	return true
end


-- USSR Hero Script
function CreateAndAssignEnemyHero1(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	--hero.portrait = "/Game/Gui/Common/Heroes/DE/de_in_16.de_in_16"
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.NoSurrender, UnitTrait.OverwhelmingAttack, UnitTrait.Vigilant}
	-- hero.modifiers = {{type = Initiative, mod = 10}, {type = Spotting, mod = 1}}
	-- hero.attack_modifiers = {{type = TargetType.Soft, mod = 10}}
	-- hero.defense_modifiers = {{type = Ground, mod = 10}}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero2(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	--hero.portrait = "/Game/Gui/Common/Heroes/DE/de_in_16.de_in_16"
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.ArtySupport}
	-- hero.modifiers = {{type = Initiative, mod = 10}, {type = Spotting, mod = 1}}
	-- hero.attack_modifiers = {{type = TargetType.Soft, mod = 10}}
	-- hero.defense_modifiers = {{type = Ground, mod = 10}}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero3(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	--hero.portrait = "/Game/Gui/Common/Heroes/DE/de_in_16.de_in_16"
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.IgnoresEntrenchment}
	-- hero.modifiers = {{type = Initiative, mod = 10}, {type = Spotting, mod = 1}}
	-- hero.attack_modifiers = {{type = TargetType.Soft, mod = 10}}
	-- hero.defense_modifiers = {{type = Ground, mod = 10}}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero4(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	--hero.portrait = "/Game/Gui/Common/Heroes/DE/de_in_16.de_in_16"
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.FearsomeReputation}
	-- hero.modifiers = {{type = Initiative, mod = 10}, {type = Spotting, mod = 1}}
	-- hero.attack_modifiers = {{type = TargetType.Soft, mod = 10}}
	-- hero.defense_modifiers = {{type = Ground, mod = 10}}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero5(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	--hero.portrait = "/Game/Gui/Common/Heroes/DE/de_in_16.de_in_16"
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.Vigilant, UnitTrait.Evasive}
	-- hero.modifiers = {{type = Initiative, mod = 10}, {type = Spotting, mod = 1}}
	-- hero.attack_modifiers = {{type = TargetType.Soft, mod = 10}}
	-- hero.defense_modifiers = {{type = Ground, mod = 10}}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function EnemyHero()
	CreateAndAssignEnemyHero1(16) -- specify id of the unit which will get this hero tank sto
	CreateAndAssignEnemyHero2(19) -- specify id of the unit which will get this hero tank
	CreateAndAssignEnemyHero3(18) -- specify id of the unit which will get this hero tank
	CreateAndAssignEnemyHero4(17) -- specify id of the unit which will get this hero at
	CreateAndAssignEnemyHero5(20) -- specify id of the unit which will get this hero tank
end


-- GB Helden
function CreateAndAssignNewHero1(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	if (unit == nil) then return end
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.TankKiller, UnitTrait.IgnoresEntrenchment}
	hero.modifiers = {{type = Initiative, mod = 1}, {type = Speed, mod = 30}}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	action.silent = true
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignNewHero2(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	if (unit == nil) then return end
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.LightningAttack}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	action.silent = true
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignNewHero3(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	if (unit == nil) then return end
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.DoubleSupport, UnitTrait.DoubleAttack}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	action.silent = true
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignNewHero4(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	if (unit == nil) then return end
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.IgnoresZOC}
	hero.modifiers = {{type = Initiative, mod = 2}, {type = Speed, mod = 20}}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	action.silent = true
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function PlayerHero()
	CreateAndAssignNewHero1(10) -- specify id of the unit which will get this hero
	CreateAndAssignNewHero2(14) -- tig
	CreateAndAssignNewHero3(12) -- arty
	CreateAndAssignNewHero4(4) -- inf
end


--german depots
GermanDepotText = NSLOCTEXT("scenario_Meridian", "GermanDepotText", "German depot uncovered!  Reinforcement has arrived near the western supply hexes.")

SupplyDrop1SW = {{2,31}}
SupplyDrop2W = {{28,33}}
SupplyDrop3NW = {{31,8}}


-- This table is initially empty, and will accumulate depots which we capture
-- Depots won't generate a bonus if we lose them and recapture for the second time
captured_depots = {}

function OnCaptureSupply1(action)
	-- If this flag was not captured by player 0 (UK), do nothing
	if action.new_owner ~= 0 then return end

	-- Don't generate reward if the same depot is captured for the second time
	if IsItemInTable(captured_depots, action.flag) then return end

	if IsItemInTable(SupplyDrop1SW, action.flag) then	
	
		player = 0
		zone = {{4,17},{5,17},{4,18},{5,18},{4,19},{5,19},{4,20},{5,20},{4,21},{5,21},{4,22},{5,22},{3,23},{4,23},{4,24}}
		units = { {"StuGIIIG", "", 3, 2200} }

		SpawnWaveUK(player, zone, units)

		MessageBox("", GermanDepotText)
	else
		return
	end
	
	-- Remember that this depot has been captured
	table.insert(captured_depots, action.flag)
end

function OnCaptureSupply2(action)
	-- If this flag was not captured by player 0 (UK), do nothing
	if action.new_owner ~= 0 then return end

	-- Don't generate reward if the same depot is captured for the second time
	if IsItemInTable(captured_depots, action.flag) then return end

	if IsItemInTable(SupplyDrop2W, action.flag) then	
	
		player = 0
		zone = {{4,17},{5,17},{4,18},{5,18},{4,19},{5,19},{4,20},{5,20},{4,21},{5,21},{4,22},{5,22},{3,23},{4,23},{4,24}}
		units = { {"PanzerIIIM", "", 5, 2200} }

		SpawnWaveUK(player, zone, units)

		MessageBox("", GermanDepotText)
	else
		return
	end
	
		-- Remember that this depot has been captured
	table.insert(captured_depots, action.flag)
end

function OnCaptureSupply3(action)
	-- If this flag was not captured by player 0 (UK), do nothing
	if action.new_owner ~= 0 then return end

	-- Don't generate reward if the same depot is captured for the second time
	if IsItemInTable(captured_depots, action.flag) then return end

	if IsItemInTable(SupplyDrop3NW, action.flag) then	
	
		local player = world:GetPlayer(1)
		
		player = 0
		zone = {{4,17},{5,17},{4,18},{5,18},{4,19},{5,19},{4,20},{5,20},{4,21},{5,21},{4,22},{5,22},{3,23},{4,23},{4,24}}
		units = { {"PantherD", "", 2, 2200} }

		SpawnWaveUK(player, zone, units)

		MessageBox("", GermanDepotText)
	else
		return
	end
	
		-- Remember that this depot has been captured
	table.insert(captured_depots, action.flag)
end

-- *** Utility functions ***
function FindItemInTable(table, item)
	for index,i in ipairs(table) do
		if i == item then
			return index 
		end
	end

	return -1
end

function IsItemInTable(table, item)
	return FindItemInTable(table, item) ~= -1
end


function SpawnWaveUK(player, zone, units)
	local owner = world:GetPlayer(player)

	local cur_hex = 1

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = true -- spawn aux units
		create_action.cost = 0 -- don't take money for units we generate
		world:Exec(create_action)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 50 }
		unit.aggressiveness = player_aggressiveness[player+1]
	end
end
