
normal_wave_turns = {8,9,18,19}

-- *** Hex zones ***
-- Spawn zones
-- When spawning a set of units, the script will scan spawn zones hex by hex, in the order specified, and spawn units in free hexes it finds
-- If the script runs out of hexes, the remainder of the units will not be spawned.
de_spawn_zone1 = {{15,1},{16,1},{17,1},{18,1},{19,1},{15,2},{16,2},{17,2},{18,2},{19,2},{15,3},{16,3},{17,3},{18,3}}
de_spawn_zone2 = {{19,33},{20,33},{19,34},{20,34},{21,34},{19,35},{20,35},{21,35},{22,35},{23,35},{20,36},{21,36},{22,36},{23,36},{24,36}}

-- *** Enemy waves ***
-- Each unit set is a list of blocks in the following format: { "<Unit ID>", "<Transport ID>", overstrength, experience }
-- If no organic transport is needed, its respective field should be empty string ("").

-- German waves
de_wave1 = { {"PanzerIIIN", "", 0, 1250}, {"Grenadier43", "SdKfz251-1", 0, 1250}, {"Pioniere43", "SdKfz250-1", 0, 1250}, {"MarderIIIH", "", 0, 1250}, {"SdKfz234-1-8Rad", "", 0, 1250}, {"105mmleFH18", "OpelBlitz", 0, 1250} }
de_wave2 = { {"PanzerIIIM", "", 0, 1250}, {"WehrInfantry43", "OpelBlitz", 0, 1250}, {"WehrInfantry43", "OpelBlitz", 0, 1250}, {"StuGIIIG", "", 0, 1250}, {"PanzerIIF", "", 0, 1250}, {"Wurfrahmen40", "", 0, 1250} }
de_wave3 = { {"PanzerIVG", "", 2, 1250}, {"WehrInfantry43", "OpelBlitz", 2, 1250}, {"WehrInfantry43", "OpelBlitz", 3, 1250}, {"PanzerIVF2", "", 1, 1250}, {"StuGIIIF-8", "", 1, 1250}, {"75mmPak40", "OpelBlitz", 2, 1250}, {"SdKfz251-16Flamm", "", 1, 1250} }
de_wave4 = { {"TigerI", "", 0, 1250}, {"Grenadier43", "SdKfz251-1", 2, 1250}, {"Grenadier43", "SdKfz251-1", 0, 1250}, {"PanzerIVF", "", 0, 1250}, {"SdKfz232-8Rad", "", 1, 1250}, {"15cmsFH18", "SdKfz251-1", 0, 1250}, {"SdKfz7-1AA", "", 1, 1250} }

-- There can be any number of waves in this list, the game will pick a random one every time
de_waves = {de_wave1, de_wave2, de_wave3, de_wave4}

-- There can be any number of zones in this list, the game will pick a random one every time
de_zones = {de_spawn_zone1, de_spawn_zone2}

function SpawnWave(player, zone, units)
	local owner = world:GetPlayer(player)

	local cur_hex = 1

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = false -- spawn core units
		create_action.cost = 0 -- don't take money for units we generate
		create_action.faction = owner.factions[1]  --set to first faction of owning player for created units
		world:Exec(create_action)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 70 }
		unit.aggressiveness = player_aggressiveness[player+1]
		
	end

end

function OnNewRound(action)
	-- On the first call, initialize the random generator
	if world.round == 1 then
		math.randomseed(world.seed)
	end

	-- Do we need to spawn a normal wave?
	local is_normal_wave = IsItemInTable(normal_wave_turns, world.round)

	-- If we don't need to spawn any normal or boss waves this turn then just finish
	if not is_normal_wave then return end

	-- Determine the player to get the next wave, always Germany 1 
	player = 1
	
	-- Determine the right zone and units to spawn
	local zone
	local units

	-- Player 1 is Germany
	zone = de_zones[ math.random( #de_zones ) ] -- random normal ru zone
	units = de_waves[ math.random( #de_waves ) ] -- random normal ru wave
		-- Now spawn this wave!
	SpawnWave(player, zone, units)
	
end
-- *** Utility functions ***
function FindItemInTable(table, item)
	for index,i in ipairs(table) do
		if i == item then
			return index 
		end
	end

	return -1
end

function IsItemInTable(table, item)
	return FindItemInTable(table, item) ~= -1
end

function EmptyCondition()
	return true
end



--enemy Hero Script
function CreateAndAssignEnemyHero1(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.NoSupply, UnitTrait.CityFighter}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero2(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.ArtySupport}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero3(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.AggressiveCounterattack}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero4(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.OverwhelmingAttack}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero5(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.LowProfile}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function EnemyHero()
	CreateAndAssignEnemyHero1(37) -- specify id of the unit which will get this hero inf
	CreateAndAssignEnemyHero2(22) -- specify id of the unit which will get this hero inf
	CreateAndAssignEnemyHero3(70) -- specify id of the unit which will get this hero arty
	CreateAndAssignEnemyHero4(92) -- specify id of the unit which will get this hero tank
	CreateAndAssignEnemyHero5(64) -- specify id of the unit which will get this hero tank
end


function CreateAndAssignNewHero1(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	if (unit == nil) then return end
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.SuperiorManeuver, UnitTrait.Butcher}
	hero.modifiers = {{type = Initiative, mod = 1}, {type = Speed, mod = 10}}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	action.silent = true
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignNewHero2(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	if (unit == nil) then return end
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.CripplingBlow}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	action.silent = true
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignNewHero3(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	if (unit == nil) then return end
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.ShockTactics, UnitTrait.NoRetaliation}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	action.silent = true
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignNewHero4(unit_id)
	local hero = NewHero()
	local unit = world:GetUnit(unit_id)
	if (unit == nil) then return end
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.extra_traits = {UnitTrait.MachineGun}
	local action = world:MakeNewHeroAction(unit.owner_id, hero)
	action.silent = true
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(unit, hero.id)
	world:Exec(assign_hero_action)
end

function PlayerHero()
	CreateAndAssignNewHero1(9) -- specify id of the unit which will get this hero  tank
	CreateAndAssignNewHero2(11) -- tank
	CreateAndAssignNewHero3(7) -- guard
	CreateAndAssignNewHero4(5) -- bomb
end

